// -*- C++ -*-

/* 
 * actor.h 
 *
 * Copyright (C) 1998 EMC Capital Management, Inc.
 *
 * Developed by Jon Trowbridge <trow@emccta.com> and
 * Havoc Pennington <hp@emccta.com>.
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

#ifndef GUPPI_ACTOR_H
#define GUPPI_ACTOR_H

#ifndef G_ENABLE_DEBUG
#define G_ENABLE_DEBUG 1
#endif

#include <gtk--draw.h>
#include <gtk--plot.h>
#include "limited.h"

/*
  Actor is a base class for the code that will paint the different types
  of plots (scatter plots, time series plots, regression lines & curves,
  etc.) by calling functions associated with a target Plot_Area.
 */

class Actor : public Limited {
public:
  Actor(Plot_Area&);
  virtual ~Actor();

  // Actually do the work of putting the pixels in the window.
  virtual void draw() const = 0;

  Gdk_Color & color() { return color_; }

protected:
  Plot_Area* target() const { return target_; }

private:
  Plot_Area* target_;
  Gdk_Color color_;
};

#endif
