// -*- C++ -*-

/* 
 * axisdoc.cc
 *
 * Copyright (C) 1998 EMC Capital Management, Inc.
 *
 * Developed by Jon Trowbridge <trow@emccta.com> and
 * Havoc Pennington <hp@emccta.com>.
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

#include "axisdoc.h"

#include <math.h>
#include <nan.h>

ColumnAxisDoc::ColumnAxisDoc(const ColumnDoc & cd) 
  : AxisDoc(0,0)
{
  set_column(cd);
  set_name(cd.get_name());
}

ColumnAxisDoc::ColumnAxisDoc() 
  : AxisDoc(0,0), cd_(0) // inits start_/stop_ to defaults
{
}

void
ColumnAxisDoc::set_column(const ColumnDoc & cd)
{
  // I am not sure how we'll know when/if the column is destroyed.
  //  fixme
  cd_ = &cd;

  double min = cd.ds().min();
  double max = cd.ds().max();
  
  if (!isnan(min)) {
    set_min(min);
    set_start(min);
  }

  if (!isnan(max)) {
    set_max(max);
    set_stop(max);
  }

  column_changed(*this);
}
