// -*- C++ -*-

/* 
 * axisdoc.h 
 *
 * Copyright (C) 1998 EMC Capital Management, Inc.
 *
 * Developed by Jon Trowbridge <trow@emccta.com> and
 * Havoc Pennington <hp@emccta.com>.
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

#ifndef GUPPI_AXISDOC_H
#define GUPPI_AXISDOC_H

#include <gtk--sig.h>
#include <libgnome/libgnome.h>

#include "serializable.h"
#include "named.h"

class AxisDoc : public Serializable,
		public Named,
		public Gtk_Signal_Base
{
public:
  AxisDoc(double min, double max) 
    : Named(_("Unlabelled Axis")),
      min_(min), max_(max),
      start_(min), stop_(max) {}
  virtual ~AxisDoc() { destroy(); }

  // fixme
  virtual string   serialize() const {return "";}
  virtual int      unserialize(const string & s) {return 0;}

  void set_start(double start) { start_ = start; size_changed(*this); }
  void set_stop(double stop) { stop_ = stop; size_changed(*this); }

  // avoids two signals.
  void set_bounds(double start, double stop) {
    start_ = start; stop_ = stop;
    size_changed(*this);
  }
  
  double get_start() const { return start_; }
  double get_stop() const { return stop_; }

  double get_min() const { return min_; }
  double get_max() const { return max_; }

  Signal0 destroy;

  Signal1<const AxisDoc &> name_changed;
  Signal1<const AxisDoc &> size_changed;

protected:
  
  void set_min(double min) { min_ = min; }
  void set_max(double max) { max_ = max; }

private:
  double min_;
  double max_;

  double start_;
  double stop_;
  
  void named_callback(const string & name) {
    name_changed(*this);
  }  
};

#include "columndoc.h"

// An axis associated with a column
class ColumnAxisDoc : public AxisDoc 
{
public:
  ColumnAxisDoc(const ColumnDoc & cd);
  ColumnAxisDoc();

  const ColumnDoc* get_column() const { return cd_; }
  void set_column(const ColumnDoc & cd);

  Signal1<const ColumnAxisDoc&> column_changed;

private:
  const ColumnDoc* cd_;
};

#endif
