// -*- C++ -*-

/* 
 * columnutils.h 
 *
 * Copyright (C) 1998 EMC Capital Management, Inc.
 *
 * Developed by Jon Trowbridge <trow@emccta.com> and
 * Havoc Pennington <hp@emccta.com>.
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

#ifndef GUPPI_COLUMNUTILS_H
#define GUPPI_COLUMNUTILS_H

#include <list>

#include "columndoc.h"

// Utility functions useful in column manipulation.

class TextImportParams {
public:
  // lazy not to use accessors, I know.

  // How to interpret the text.
  //  If true, each column of text is a ColumnDoc
  //   (i.e. the bytestream has ColumnDoc's interleaved)
  //  If false, each row of text is a ColumnDoc
  //   (i.e. the bytestream has ColumnDoc's sequentially)
  //  'column' and 'row' are defined by the below separators.
  bool   columnar;
  string element_separator;
  string row_separator;

  // this could be computed, but it is also allowed to choose fewer
  // elements or columns than actually exist.
  guint  num_elements;
  
  // if false, ignore the num_elements member.
  bool   limit_elements;

  // was hoping to avoid this function complication

  TextImportParams() : columnar(true), element_separator(""),
    row_separator(""), num_elements(0), types(0), num_columns(0)
    {}

  ~TextImportParams() { if (types != 0) delete types; }

  // bleh, too big to inline. fixme.
  void set_columns(size_t n) {
    size_t old_n = num_columns;
    ColumnDoc::DataType* old_types = types;

    num_columns = n;
    if (n > 0) {
      types = new ColumnDoc::DataType[n];
    }
    else {
      types = 0;
    }

    size_t i = 0;
    while ( i < old_n ) {
      if ( i == n ) break; // last one was deleted
      types[i] = old_types[i];
      ++i;
    }

    if (old_types != 0) delete old_types;
  }

  size_t get_columns() const { return num_columns; }

  ColumnDoc::DataType get_type(size_t n) const {
    if (types != 0) {
      return types[n];
    }
    else return ColumnDoc::UNKNOWN;
  }

  void set_type(size_t col, ColumnDoc::DataType t) {
    if (col < num_columns) {
      types[col] = t;
    }
#ifdef GNOME_ENABLE_DEBUG
    else g_warning("No such column to set the type of!");
#endif
  }

private:
  // array of types.
  ColumnDoc::DataType* types;
  guint  num_columns;
};

// import text, using the supplied parameters. import_text will be
//  very dumb about it; you ask for it, you got it. so don't
//  get the parameters wrong. 
list<ColumnDoc*> * import_text(const string & text, 
			       const TextImportParams & params);

// fill params with our best guess - the site of intelligence.
void               guess_import_parameters(const string & text,
					   TextImportParams & params);

#endif
