// -*- C++ -*-

/* 
 * columnwindow.cc 
 *
 * Copyright (C) 1998 EMC Capital Management, Inc.
 *
 * Developed by Jon Trowbridge <trow@emccta.com> and
 * Havoc Pennington <hp@emccta.com>.
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

#include "columnwindow.h"
#include "guppi.h"


class ColumnWindowImpl : public Gtk_Notebook {
public:
  ColumnWindowImpl(ColumnDoc& cd, ColumnWindow& dialog);
  ~ColumnWindowImpl();

private:
  ColumnDoc& cd_;
  ColumnWindow& dialog_;

  Gtk_Label title_;

  Gtk_Entry name_entry_;

  Gtk_VBox  statsbox_;
  Gtk_Label statslabel_;
  Gtk_CList stats_;
  
  Gtk_VBox  databox_;
  Gtk_Label datalabel_;
  Gtk_CList data_;

  void display_name(const ColumnDoc & cd);
  void display_data(const DataSet& ds);
  
  void doc_destroyed(const ColumnDoc & cd) { dialog_.self_munch(); }

  void name_entry_activate();
#if 0
  void name_entry_focus_out(GdkEventFocus* ef) {
    name_entry_activate();
  }
#endif

  typedef double (DataSet::*statfunc)() const;

  void fill_statistic(gchar* text[], const DataSet& ds, 
		      statfunc f,
		      gchar* name);
};

ColumnWindow::ColumnWindow(ColumnDoc& cd)
  :  Gnome_Dialog("", GNOME_STOCK_BUTTON_CLOSE, NULL), 
     impl_(new ColumnWindowImpl(cd,*this))
{
  set_policy(FALSE,TRUE,FALSE);
}

ColumnWindow::~ColumnWindow()
{

}

void
ColumnWindow::self_munch()
{
  big_fish.munch(this);
}


ColumnWindowImpl::ColumnWindowImpl(ColumnDoc& cd, ColumnWindow& dialog) :
  //  Gtk_HBox(FALSE,GNOME_PAD),
  cd_(cd),
  dialog_(dialog),
  title_(_("Column Information")),
  name_entry_(),
  statsbox_(FALSE,GNOME_PAD_SMALL),
  statslabel_(_("Statistics")),
  stats_(2),
  databox_(FALSE,GNOME_PAD_SMALL),
  datalabel_(_("Data")),
  data_(2)
{
  connect_to_method(cd.destroy,
		    this,
		    &ColumnWindowImpl::doc_destroyed);

  dialog.vbox()->pack_start(&title_,FALSE,FALSE,GNOME_PAD_SMALL);
  title_.show();
  dialog.vbox()->pack_start(&name_entry_,FALSE,FALSE,GNOME_PAD_SMALL);
  name_entry_.show();

  dialog.vbox()->pack_start(this,TRUE,TRUE,0);

  connect_to_method(name_entry_.activate,
		    this,
		    &ColumnWindowImpl::name_entry_activate);

  data_.set_usize(200,100);
  data_.set_column_width(0,80);
  data_.set_column_width(1,120);
  data_.set_column_title(0,_("Observation"));
  data_.set_column_title(1,_("Value"));
  data_.column_titles_show();
  data_.column_titles_passive();
  data_.set_policy(GTK_POLICY_AUTOMATIC,GTK_POLICY_AUTOMATIC);
  //  databox_.pack_start(&datalabel_,FALSE,FALSE);
  databox_.pack_start(&data_,TRUE,TRUE);
  append_page(&databox_,&datalabel_);

  stats_.set_usize(280,100);
  stats_.set_column_title(0,_("Statistic"));
  stats_.set_column_width(0,150);
  stats_.set_column_title(1,_("Value"));
  stats_.set_column_width(1,80);
  stats_.column_titles_show();
  stats_.set_policy(GTK_POLICY_AUTOMATIC,GTK_POLICY_AUTOMATIC);
  //  statsbox_.pack_start(&statslabel_,FALSE,FALSE);
  statsbox_.pack_start(&stats_,TRUE,TRUE);
  append_page(&statsbox_,&statslabel_);

  display_name(cd);
  display_data(cd.ds());

  connect_to_method(cd.name_changed, this, &ColumnWindowImpl::display_name);
  connect_to_method(cd.data_changed, this, &ColumnWindowImpl::display_data);

  show_all();
}

ColumnWindowImpl::~ColumnWindowImpl()
{

}

void 
ColumnWindowImpl::fill_statistic(gchar* text[], const DataSet& ds, 
				 statfunc func,
				 gchar* name)
{
  static gchar buf[100];

  g_snprintf(buf,100,"%f",(ds.*func)());
  text[0] = name;
  text[1] = buf;
}

void
ColumnWindowImpl::display_data(const DataSet & ds)
{
  // First fill in the data box.
  data_.clear();

  size_t i = 0;
  size_t n = ds.size();

  while ( i < n ) {
    gchar buf1[100];
    // start counting at 1
    g_snprintf(buf1,100,"%u",i+1);
    // this will need formatting according
    //  to data type.
    gchar buf2[100];
    g_snprintf(buf2,100,"%.4f",ds.data(i));
    // bah
    gchar* text[2];
    text[0] = &buf1[0];
    text[1] = &buf2[0];
    
    data_.append(text);
    ++i;
  }

  // now the statistical info.

  stats_.clear();

  gchar* text[2];

  text[0] = _("N");
  gchar buf[100];
  g_snprintf(buf,100,"%u",ds.size());
  text[1] = buf;
  stats_.append(text);

  fill_statistic(text,ds,&DataSet::min,_("Minimum"));
  stats_.append(text);

  fill_statistic(text,ds,&DataSet::max,_("Maximum"));
  stats_.append(text);

  fill_statistic(text,ds,&DataSet::range,_("Range"));
  stats_.append(text);

  fill_statistic(text,ds,&DataSet::sum,_("Sum"));
  stats_.append(text);

  fill_statistic(text,ds,&DataSet::mean,_("Mean"));
  stats_.append(text);

  fill_statistic(text,ds,&DataSet::var,_("Variance"));
  stats_.append(text);

  fill_statistic(text,ds,&DataSet::sdev,_("Population Std Deviation"));
  stats_.append(text);

  fill_statistic(text,ds,&DataSet::sdevs,_("Sample Std Deviation"));
  stats_.append(text);

  fill_statistic(text,ds,&DataSet::med,_("Median"));
  stats_.append(text);

  fill_statistic(text,ds,&DataSet::q1,_("Q1"));
  stats_.append(text);
  fill_statistic(text,ds,&DataSet::q3,_("Q3"));
  stats_.append(text);

  fill_statistic(text,ds,&DataSet::iqr,_("IQR"));
  stats_.append(text);
  fill_statistic(text,ds,&DataSet::gmean,_("Geometric Mean"));
  stats_.append(text);

  fill_statistic(text,ds,&DataSet::hmean,_("Harmonic Mean"));
  stats_.append(text);
  fill_statistic(text,ds,&DataSet::rms,_("Root Mean Square"));
  stats_.append(text);
  fill_statistic(text,ds,&DataSet::meandev,_("Mean Deviation"));
  stats_.append(text);
  fill_statistic(text,ds,&DataSet::meddev,_("Median Deviation"));
  stats_.append(text);
  fill_statistic(text,ds,&DataSet::kurtosis,_("Kurtosis"));
  stats_.append(text);
  fill_statistic(text,ds,&DataSet::skewness,_("Skewness"));
  stats_.append(text);
  fill_statistic(text,ds,&DataSet::excess_kurtosis,_("Excess Kurtosis"));
  stats_.append(text);
  fill_statistic(text,ds,&DataSet::momental_skewness,_("Momental Skewness"));
  stats_.append(text);
}

void
ColumnWindowImpl::display_name(const ColumnDoc & cd)
{
  name_entry_.set_text(cd.get_name().c_str());
  dialog_.set_title(cd.get_name().c_str());
}

void
ColumnWindowImpl::name_entry_activate()
{
  // fixme make sure the returned string is static
  cd_.set_name(name_entry_.get_text());
}

