// -*- C++ -*-

/* 
 * gtkaxis.cc 
 *
 * Copyright (C) 1998 EMC Capital Management, Inc.
 *
 * Developed by Jon Trowbridge <trow@emccta.com> and
 * Havoc Pennington <hp@emccta.com>.
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

#include "gtkaxis.h"

Gtk_Axis::Gtk_Axis(Axis_Base::alignment_t a) 
  : Axis_Base(a)
{
  set_events(get_events() | GDK_POINTER_MOTION_HINT_MASK);
}

Gtk_Axis::~Gtk_Axis()
{
  
}

gint 
Gtk_Axis::button_press_event_impl(GdkEventButton* be)
{
#ifdef GNOME_ENABLE_DEBUG
  g_print("Gtk_Axis got button press\n");
#endif
  return FALSE;
}

gint 
Gtk_Axis::configure_event_impl(GdkEventConfigure* ce)
{
  // The order of events here is very important.
  new_pixmap(ce);
  set_drawable(buffer());
  set_secondary_drawable(screen());

  redraw();

  return TRUE;
}

gint 
Gtk_Axis::motion_notify_event_impl(GdkEventMotion* me)
{
  gint x,y;
  GdkModifierType state;

  if (me->is_hint)
    gdk_window_get_pointer (me->window, &x, &y, &state);
  else
    {
      x = (gint) me->x;
      y = (gint) me->y;
      state = (GdkModifierType) me->state;
    }

#ifdef GNOME_ENABLE_DEBUG
  g_print("Gtk_Axis::motion_notify_event_impl: %d,%d\n", x,y);
#endif

  // does nothing; haven't decided what it should do.
  if (state & GDK_BUTTON1_MASK) {

    return FALSE;
  }

  return FALSE;
}

void
Gtk_Axis::redraw()
{
  set_drawing_target(this);

  //  gdk_window_clear(get_window());

  gc_save();
  // Eventually the user can set this.
  gc()->set_foreground("white");
  
  // should be done by setting widget background
  draw_rectangle(true,0,0,Gdk_Draw::width(),Gdk_Draw::height());

  gc_restore();
}

