// -*- C++ -*-

/* 
 * guppidoc.cc
 *
 * Copyright (C) 1998 EMC Capital Management, Inc.
 *
 * Developed by Jon Trowbridge <trow@emccta.com> and
 * Havoc Pennington <hp@emccta.com>.
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */


#include "guppidoc.h"

#include <libgnome/libgnome.h>

GuppiDoc::GuppiDoc() : Named(_("Untitled Data Set"))
{
  set_gnome_icon("gnome-default.png");
}

GuppiDoc::~GuppiDoc()
{
  destroy();
}

void 
GuppiDoc::add_column(ColumnDoc & c)
{
  // fixme check for another column with the same name
  columns_.push_back(&c);
#if 0 // this mysteriously segv's
  connect_to_method(c.data_changed,this,
		    &GuppiDoc::column_data_changed);
#endif
  column_added(c);
}

void 
GuppiDoc::delete_column(ColumnDoc* c)
{
  ColumnDoc* found = 0;

  list<ColumnDoc*>::iterator i = columns_.begin();
  while ( i != columns_.end() ) {
    if (c == *i) {
      found = c;
      columns_.erase(i);
      break;
    }
    ++i;
  }

  if (found != 0) {
    column_deleted(*c);
    not_saved();
    delete c;
  }
  else {
    g_warning("Column not found in GuppiDoc!\n");
  }
}

void
GuppiDoc::column_data_changed(const DataSet & ds)
{
  not_saved();
}

void 
GuppiDoc::add_plot(PlotDoc & p)
{
  // fixme check for another column with the same name
  plots_.push_back(&p);
  plot_added(p);
}

void 
GuppiDoc::delete_plot(PlotDoc* pd)
{
  PlotDoc* found = 0;

  list<PlotDoc*>::iterator i = plots_.begin();
  while ( i != plots_.end() ) {
    if (pd == *i) {
      plots_.erase(i);
      found = pd;
      break;
    }
    ++i;
  }

  if (found != 0) {
    plot_deleted(*pd);
    not_saved();
    delete pd;
  }
  else {
    g_warning("Plot not found in GuppiDoc!\n");
  }
}

string
GuppiDoc::serialize() const
{
  string output; 

  output = string("{\n") + string("GuppiDoc\n") + get_name() + '\n';

  const_column_iterator i = columns_begin();
  while ( i != columns_end() ) {
    output += "{\n";
    output += (*i)->serialize();
    output += "}\n";    
    ++i;
  }

  // fixme same for plots

  output += "}\n";

  return output;
}

int
GuppiDoc::unserialize(const string & s)
{
#if 0
  int pos = 0;
  int size = s.size();


  // Loop invariants: 
  //  pos is the start of a line at the beginning of the loop.
  //  newline is immediately set to the end of the same line
  //  the file format has a single token per line

  while ( pos < size ) {
    // Advance pos past any white space or newlines
    //  (note that this could interfere with our newline handling
    //   if we do things wrong)
    while (isspace(s[pos])) ++pos;

    int newline = s.find_first_of('\n',pos);
    int len = newline - pos;

    g_print("pos: %d newline: %d token: '%s'\n",
	    pos, newline, s.substr(pos,len).c_str());

    // Start of an object
    if (s[pos] == '{') {
      pos = s.find_first_of('\n',pos) + 1;
    }

    // Start of a ColumnDoc
    else if (s.substr(pos,len) == "ColumnDoc") {
      ColumnDoc* cd = new ColumnDoc;
      
      int endofdoc = s.find_first_of('}',pos);
      // skip the type tag
      int startofdoc = s.find_first_of('\n',newline) + 1;
      int doclen = endofdoc - startofdoc;

      // + 1 so we don't send the first newline to the ColumnDoc
      int result = cd->unserialize(s.substr(startofdoc,doclen));
      if (result == -1) {
	delete cd;
	return -1;
      }
      else {
	add_column(*cd);
	pos = s.find_first_of('\n',endofdoc) + 1;
      }
    }

    // Start of the GuppiDoc
    else if (s.substr(pos,len) == "GuppiDoc") {
      int endofname = s.find_first_of('\n',newline+1);
      set_name(s.substr(newline+1,endofname-newline-1));

      // one past \n
      pos = endofname + 1;
    }

    // End of an object
    else if (s[pos] == '}') {
      // advance to next token.
      pos = newline + 1;
    }

    // Dunno what it is.
    else {
      g_warning("Didn't understand this token: '%s'", 
		s.substr(pos,newline-pos).c_str());
      return -1;
    }
  }
  return pos;
#endif
  return 0;
}

guint GuppiDoc::num_docs_;

