// -*- C++ -*-

/* 
 * lineactor.cc
 *
 * Copyright (C) 1998 EMC Capital Management, Inc.
 *
 * Developed by Jon Trowbridge <trow@emccta.com> and
 * Havoc Pennington <hp@emccta.com>.
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

#include <nan.h>

#include "lineactor.h"

#include "mathutil.h"

LineActor::LineActor(Plot_Area & pa) : Actor(pa)
{
  
}

LineActor::~LineActor()
{

}

void
LineActor::set_x_bounds(double min, double max)
{ 
  Limited::set_x_bounds(min,max);

  // Only exactly equal to 0 counts here, no?
  if ( m_ != 0.0 ) {
    // set_y_bounds will flip these if needed to make 
    //  them min/max
    Limited::set_y_bounds(y(x_min()),y(x_max()));
  }
  else {
    Limited::set_y_bounds(b_,b_);
  }
}

void
LineActor::set_y_bounds(double min, double max)
{ 
  Limited::set_y_bounds(min,max);
  if ( m_ != 0.0 ) {
    // flips min and max if needed.
    Limited::set_x_bounds(x(y_min()),x(y_max()));
  }
  // If m == 0.0 there's no way to come up with 
  //  a reasonable set of y bounds due to divide by 0
}

void
LineActor::draw() const
{
  g_return_if_fail(target() != 0);

#if 0
  // This would be much simpler if x_min_, y_min_ was 
  //  guaranteed to be an ordered pair.

  // not sure my terminology is mathematically correct
  gdouble f_of_x_min, f_of_y_min,
    f_of_x_max, f_of_y_max;

  f_of_x_min = y(x_min());
  f_of_y_min = x(y_min());
  f_of_x_max = y(x_max());
  f_of_y_max = x(y_max());

  gdouble x0,x1,y0,y1;

  // Choose the innermost ordered pair, either (x_min_, f_of_x_min)
  //  or (f_of_y_min, y_min_) and same for max

  // The highest X value on the left of the graph is 
  //  the innermost ordered pair, and the lowest on the
  //  right. If the values are equal we don't care
  //  which condition succeeds.

  if (x_min() > f_of_x_min) {
    x0 = x_min();
    y0 = f_of_x_min;
  }
  else {
    x0 = f_of_y_min;
    y0 = y_min();
  }

  if (f_of_y_max > x_max()) {
    x1 = x_max();
    y1 = f_of_x_max;
  }
  else {
    x1 = f_of_y_max;
    y1 = y_max();
  }
#endif

  gdouble x0,x1,y0,y1;

  x0 = x_min();
  y0 = y(x_min());
  x1 = x_max();
  y1 = y(x_max());

  g_print("LineActor: draw %f,%f to %f,%f\n",x0,y0,x1,y1);

  target()->draw_line(x0,y0,x1,y1);
}



