// -*- C++ -*-

/* 
 * lineactor.h 
 *
 * Copyright (C) 1998 EMC Capital Management, Inc.
 *
 * Developed by Jon Trowbridge <trow@emccta.com> and
 * Havoc Pennington <hp@emccta.com>.
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

#ifndef GUPPI_LINEACTOR_H
#define GUPPI_LINEACTOR_H

#include "actor.h"
#include <math.h>
#include <nan.h>

class LineActor : public Actor {
public:
  LineActor(Plot_Area&);
  virtual ~LineActor();

  void set_equation(gdouble slope, gdouble intercept) { 
    m_ = slope; b_ = intercept; 
  }

  // These are overridden to set the other bound based on
  //  the bound being set. Perhaps a bad idea. ???
  virtual void set_x_bounds(gdouble x0,gdouble x1);
  virtual void set_y_bounds(gdouble y0,gdouble y1);

  // Actually do the work of putting the pixels in the window.
  virtual void draw() const;
  
protected:
  virtual gdouble y(gdouble x) const { return (m_ * x) + b_; }
  // m_ == 0 should be special cased elsewhere so this doesn't happen.
  // Maybe need to check within a small range, but can you get an FPU
  //  exception merely for dividing by a very small number?
  virtual gdouble x(gdouble y) const {
    g_return_val_if_fail(m_ != 0.0, NAN); return (y-b_)/m_; 
  }

private:
  gdouble m_, b_;
};

#endif
