// -*- C++ -*-

/* 
 * muncher.h 
 *
 * Copyright (C) 1998 EMC Capital Management, Inc.
 *
 * Developed by Jon Trowbridge <trow@emccta.com> and
 * Havoc Pennington <hp@emccta.com>.
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

#ifndef GUPPI_MUNCHER_H
#define GUPPI_MUNCHER_H

#include <gtk--.h>

template <class T>
class Muncher : public Gtk_Signal_Base {
public:
  Muncher() : idle_connected_(false) {}
  virtual ~Muncher() {}
  
  void munch(T* victim);
  
private:
  gint munchidle();  
  list<T*> munchlist_;
  bool idle_connected_;
};


// I hate having implementations in header files.

template <class T>
void Muncher<T>::munch(T* victim) 
{
  g_return_if_fail(victim != 0);
  
  if (!idle_connected_) {
    connect_to_method(Gtk_Main::idle(), 
		      this,
		      &Muncher::munchidle);
    idle_connected_ = true;
  }
  munchlist_.push_back(victim);

  g_print("Pushed %p onto munchlist\n", victim);
}

template <class T>
gint Muncher<T>::munchidle()
{
  g_assert(idle_connected_);
  
  list<T*>::iterator i = munchlist_.begin();
  
  while (i != munchlist_.end()) {
    g_print("About to munch %p\n", *i);
    delete *i;
    g_print("Munched %p\n", *i);
    ++i;
  }
  munchlist_.clear();
  
  idle_connected_ = false;
  return 0; // causes disconnect
}


#endif
