// -*- C++ -*-

/* 
 * plotdoc.h 
 *
 * Copyright (C) 1998 EMC Capital Management, Inc.
 *
 * Developed by Jon Trowbridge <trow@emccta.com> and
 * Havoc Pennington <hp@emccta.com>.
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

#ifndef GUPPI_PLOTDOC_H
#define GUPPI_PLOTDOC_H

// A "document" which records information necessary to reconstruct
//  a particular plot. Does not store the DataColumn, since the
//  owning GuppiDoc will do that.

// PlotWindow needs to be changed to act as the "view" for one of these.
//  (which implies the actors are a "view" for the layers)

#include <list>

#include "plot.h"

#include <gtk--.h>  // only for make_editor()
#include "named.h"
#include "serializable.h"
#include "iconed.h"

class GuppiDoc;

class PlotDoc : public Named,
		public Iconed,
		public Serializable,
		public Gtk_Signal_Base
{
public:

  PlotDoc();
  virtual ~PlotDoc();

  // Eventually the virtual functions' functionality needs to be part
  // of a dynamically loadable module system. i.e. new plot types
  // should be on-the-fly loadable, rather than compiled-in subclasses

  // fixme I think these still have implementaitons
  virtual string   serialize() const = 0;
  virtual int      unserialize(const string & s) = 0;

  virtual Gtk_Widget* make_editor(GuppiDoc& gd) = 0;
  virtual Plot*       make_plot  () const       = 0;

  // Allow connections to const objects with mutable
  mutable Signal0 destroy;

  mutable Signal1<const PlotDoc &> name_changed;
  mutable Signal1<const PlotDoc &> icon_changed;

private:
  void named_callback(const string & name) {
    name_changed(*this);
  }

  void iconed_callback(const string & name) {
    icon_changed(*this);
  }
};


#endif
