// -*- C++ -*-

/* 
 * plotedit.cc
 *
 * Copyright (C) 1998 EMC Capital Management, Inc.
 *
 * Developed by Jon Trowbridge <trow@emccta.com> and
 * Havoc Pennington <hp@emccta.com>.
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

#include <gnome--.h>

#include "plotedit.h"
#include "guppi.h"

#include "columndoc.h"

#include "scatteredit.h"

class PlotEditImpl : public Gtk_Frame {
public:
  PlotEditImpl(PlotDoc & pd, GuppiDoc & gd, PlotEdit & window);
  ~PlotEditImpl();

protected:
  void display(); // Do the whole display

  void display_icon(const PlotDoc& pd);
  void display_name(const PlotDoc& pd);

  void apply(); // apply changes to the PlotDoc.

  void name_entry_activate();

#if 0
  void name_entry_focus_out(GdkEventFocus* ef) {
    name_entry_activate();
  }
#endif

private:
  PlotDoc& pd_;   // to apply changes.
  GuppiDoc& gd_; // to get the columns.

  PlotEdit & window_;

  Gtk_Frame frame_;
  Gtk_VBox main_box_;

  Gnome_Pixmap icon_;
  Gtk_Entry    name_entry_;
  Gtk_HBox     top_box_;

  Gtk_Widget*  editor_;
};

PlotEdit::PlotEdit(PlotDoc & pd, GuppiDoc & gd) :
  Gtk_Window(GTK_WINDOW_TOPLEVEL),
  impl_(new PlotEditImpl(pd,gd,*this))

{
  border_width(GNOME_PAD_SMALL);
  this->add(impl_);

  // This also handles the GuppiDoc being destroyed,
  //  as long as the GuppiDoc contains the PlotDoc.
  connect_to_method(pd.destroy,
		    this,
		    &PlotEdit::self_munch);

  set_policy(TRUE,TRUE,TRUE);
}

PlotEdit::~PlotEdit()
{ 
  impl_->hide(); 
  delete impl_; 
};

void
PlotEdit::self_munch()
{
  big_fish.munch(this);
}


PlotEditImpl::PlotEditImpl(PlotDoc & pd, GuppiDoc & gd, PlotEdit & window) :
  pd_(pd), gd_(gd), window_(window),

  frame_(),
  main_box_(FALSE,GNOME_PAD), 

  icon_(), name_entry_(),

  top_box_(FALSE,GNOME_PAD), 
  editor_(0)
{
  set_shadow_type(GTK_SHADOW_ETCHED_IN);
    
  add(&main_box_);  

  main_box_.pack_start(&top_box_,FALSE,FALSE);
  main_box_.border_width(GNOME_PAD);
  
  top_box_.pack_start(&icon_,FALSE,FALSE);
  top_box_.pack_start(&name_entry_,TRUE,TRUE);

  connect_to_method(name_entry_.activate,
		    this,
		    &PlotEditImpl::name_entry_activate);


  editor_ = pd_.make_editor(gd_);

  main_box_.pack_start(editor_,TRUE,TRUE);

  display();

  show_all();
}

PlotEditImpl::~PlotEditImpl() 
{
  editor_->hide();
  delete editor_;
}

void
PlotEditImpl::display()
{
  display_icon(pd_);
  display_name(pd_);
}

void
PlotEditImpl::display_icon(const PlotDoc& pd)
{
  icon_.load(pd.get_icon().c_str());
}

void
PlotEditImpl::display_name(const PlotDoc& pd)
{
  name_entry_.set_text(pd.get_name().c_str());
  window_.set_title(pd.get_name().c_str());
}

void
PlotEditImpl::name_entry_activate()
{
  // fixme make sure this isn't a mem leak
  pd_.set_name(name_entry_.get_text());
}

void
PlotEditImpl::apply()
{
  g_warning("PlotEditImpl::apply() not implemented");
}
