// -*- C++ -*-

/* 
 * serializable.h 
 *
 * Copyright (C) 1998 EMC Capital Management, Inc.
 *
 * Developed by Jon Trowbridge <trow@emccta.com> and
 * Havoc Pennington <hp@emccta.com>.
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

#ifndef GUPPI_SERIALIZABLE_H
#define GUPPI_SERIALIZABLE_H

// This interface reflects an object that can be saved and 
//  loaded as a string of text. The idea is stolen from Java.

// If each object has this method, a composite object like a GuppiDoc
//  can be loaded and saved with recursive calls. Or sent across the 
//  net, or whatever.

#include <string>

class Serializable {
public:

  // Make into a string suitable for storing in a file
  virtual string serialize() const = 0;

  // Construct the object from the stored string.
  //  Return the first unprocessed index, or -1 
  //  on error.
  virtual int     unserialize(const string & s) = 0;

};


#endif
