// -*- C++ -*-

/* 
 * unit.cc
 *
 * Copyright (C) 1998 EMC Capital Management, Inc.
 *
 * Developed by Jon Trowbridge <trow@emccta.com> and
 * Havoc Pennington <hp@emccta.com>.
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

#include "unit.h"
#include <string>

// obviously this level of abstraction is fairly
//  useless for the moment, but one might want 
//  to add e.g. a default formatter
class Unit {
  friend class UnitManager;

  Unit(const char* name) : 
    name_(name) {}
  ~Unit() {}

  string name_;
};

UnitManager::UnitManager()
{
}

UnitManager::~UnitManager()
{
  map<guint,Unit*>::iterator i = units_.begin();

  while ( i != units_.end() ) {
    delete (*i).second;
    ++i;
  }
}

guint 
UnitManager::get_id(const char* name) const
{
  return g_str_hash(name);
}

const char*
UnitManager::get_name(guint id) const
{
  Unit* u = find_unit(id);

  if (u != 0) {
    return u->name_.c_str();
  }
  else {
    return 0;
  }
}

void 
UnitManager::set_name(guint id, const char* name)
{
  Unit* u = find_unit(id);

  if (u != 0) {
    u->name_ = name;
  }
  else {
    g_warning("Couldn't set unit name");
  }
}

guint 
UnitManager::register_unit(const char* name)
{
  guint new_id = g_str_hash(name);

  // The safety check basically doubles the
  // cost of the function; should be worth it
  //  though.

  if (id_exists(new_id)) {
    g_warning("Unit id %d already exists!", new_id);
    return 0;
  }
  else {
    units_[new_id] = new Unit(name);
    unit_added(new_id);
    return new_id;
  }
}

void 
UnitManager::unregister_unit(guint id)
{
  map<guint,Unit*>::iterator i = units_.find(id);

  if (i == units_.end()) {
    g_warning("Unit ID %d not found to unregister", id);
  }
  else {
    unit_deleted(id);
    delete ((*i).second);
    units_.erase(i);
  }
}

bool 
UnitManager::id_exists(guint id) const
{
  return (units_.find(id) != units_.end());
}

Unit* 
UnitManager::find_unit(guint id) const
{
  map<guint,Unit*>::const_iterator i = units_.find(id);

  if (i != units_.end()) {
    return (*i).second;
  }
  else {
    g_warning("UnitManager: No such id %d!", id);
    return 0;
  }
}
