// -*- C++ -*-

/* 
 * unit.h 
 *
 * Copyright (C) 1998 EMC Capital Management, Inc.
 *
 * Developed by Jon Trowbridge <trow@emccta.com> and
 * Havoc Pennington <hp@emccta.com>.
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

#ifndef GUPPI_UNIT_H
#define GUPPI_UNIT_H

#include <map>
#include <gtk--sig.h>
#include <glib.h>

//#include "format.h"
#include "util.h"

class Unit;

class UnitManager : public Gtk_Signal_Base
{
public:
  UnitManager();
  ~UnitManager();

  // Allow iteration over all existing unit ID's.

  typedef first_iterator<guint,Unit*> iterator;
  typedef const_first_iterator<guint,Unit*> const_iterator;

  iterator begin() { return iterator(units_.begin()); }
  const_iterator begin() const { return const_iterator(units_.begin()); }

  iterator end() { return iterator(units_.end()); }
  const_iterator end() const { return const_iterator(units_.end()); }

  
  bool id_exists(guint id) const;

  // be sure the id exists before doing get/set.

  guint get_id(const char* name) const;

  const char* get_name(guint id) const;
  void set_name(guint id, const char* name);

  // Return the id of the new unit
  guint register_unit(const char* name);
  void unregister_unit(guint id);

  // emitted on (un)registration
  Signal1<guint> unit_added; 
  Signal1<guint> unit_deleted;

private:  
  map<guint,Unit*> units_;

  Unit* find_unit(guint id) const;
};


#endif
